unit uController.Albums;

interface

uses
  MVCFramework, MVCFramework.Commons, MVCFramework.Serializer.Commons;

type

  [MVCPath('/api')]
  TControllerAlbums = class(TMVCController) 
  public
    [MVCPath]
    [MVCHTTPMethod([httpGET])]
    procedure Index;
  private
    procedure GetCustomers;

  protected
    procedure OnBeforeAction(Context: TWebContext; const AActionName: string; var Handled: Boolean); override;
    procedure OnAfterAction(Context: TWebContext; const AActionName: string); override;

  public
    [MVCPath('/albums')]
    [MVCHTTPMethod([httpGET])]
    procedure GetAlbums;

    [MVCPath('/albums/($id)')]
    [MVCHTTPMethod([httpGET])]
    procedure GetAlbum(id: Integer);

    [MVCPath('/albums')]
    [MVCHTTPMethod([httpPOST])]
    procedure AddAlbum;

    [MVCPath('/albums/($id)')]
    [MVCHTTPMethod([httpPUT])]
    procedure UpdateAlbum(id: Integer);

    [MVCPath('/albums/($id)')]
    [MVCHTTPMethod([httpDELETE])]
    procedure DeleteAlbum(id: Integer);

  end;

implementation

uses
  System.SysUtils, MVCFramework.Logger, System.StrUtils, uDMConexion,
  MVCFramework.dataset.Utils;

procedure TControllerAlbums.Index;
begin
  //use Context property to access to the HTTP request and response 
  Render('Hello DelphiMVCFramework World');
end;


procedure TControllerAlbums.OnAfterAction(Context: TWebContext; const AActionName: string); 
begin
  { Executed after each action }
  inherited;
end;

procedure TControllerAlbums.OnBeforeAction(Context: TWebContext; const AActionName: string; var Handled: Boolean);
begin
  { Executed before each action
    if handled is true (or an exception is raised) the actual
    action will not be called }
  inherited;
end;

//Sample CRUD Actions for a "Customer" entity
procedure TControllerAlbums.GetCustomers;
begin
  //todo: render a list of customers
end;

procedure TControllerAlbums.GetAlbums;
begin
  dmConexion.qAlbums.Open;
  Render(dmConexion.qAlbums, False);
end;

procedure TControllerAlbums.GetAlbum(id: Integer);
begin
  dmConexion.qAlbums.Open('SELECT * FROM albums WHERE AlbumId=?',[id]);
  Render(dmConexion.qAlbums, False, TMVCDatasetSerializationType.dstSingleRecord);
end;

procedure TControllerAlbums.AddAlbum;
begin
  dmConexion.qAlbums.Close;
  dmConexion.qAlbums.CachedUpdates := True;
  dmConexion.qAlbums.Open;
  dmConexion.qAlbums.Insert;
  dmConexion.qAlbums.LoadFromJSONObjectString(Context.Request.Body);
  dmConexion.qAlbums.ApplyUpdates;

  Render('Insertado correctamente el id = ' + dmConexion.qAlbums.FieldByName('AlbumId').AsString);
end;

procedure TControllerAlbums.UpdateAlbum(id: Integer);
begin
  dmConexion.qAlbums.Close;
  dmConexion.qAlbums.CachedUpdates := True;
  dmConexion.qAlbums.Open('SELECT * FROM albums WHERE AlbumId=?',[id]);
  if (dmConexion.qAlbums.IsEmpty) then
    ResponseStatus(HTTP_STATUS.NotFound, ' El album no se ha encontrado')
  else begin
    dmConexion.qAlbums.Edit;
    dmConexion.qAlbums.LoadFromJSONObjectString(Context.Request.Body);
    dmConexion.qAlbums.ApplyUpdates;
    ResponseStatus(HTTP_STATUS.OK, ' Se ha actualixado el album con Id = ' + IntToStr(id));
  end;
end;

procedure TControllerAlbums.DeleteAlbum(id: Integer);
begin
  dmConexion.qAlbums.Close;
  dmConexion.qAlbums.CachedUpdates := True;
  dmConexion.qAlbums.Open('SELECT * FROM albums WHERE AlbumId=?',[id]);
  dmConexion.qAlbums.Delete;
  dmConexion.qAlbums.ApplyUpdates;
  ResponseStatus(HTTP_STATUS.OK, ' El album se ha borrado correctamente');
end;



end.
